-- $Id: DualSpeccer.lua 11 2009-08-05 15:02:44Z jeffrey0 $
DualSpeccer = LibStub("AceAddon-3.0"):NewAddon("DualSpeccer", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0")

local groupNames = {"primary", "secondary"}
local groupNamesCaps = {"Primary", "Secondary"}

local primarySpecSpellName = GetSpellInfo(63645)
local secondarySpecSpellName = GetSpellInfo(63644)

local specCache = {}

local filtering = false
local lastSwitchStartTime = 0

local rightClickMenu = {
	{
		text = "Hide switch spam",
		func = function() DualSpeccer.db.profile.filterSpam = not DualSpeccer.db.profile.filterSpam; DualSpeccer:Update() end,
		checked = function() return DualSpeccer.db.profile.filterSpam end,
		tooltipTitle = "Hide switch spam",
		tooltipText = "Hides the spell learned/unlearned messages when swapping specs."
	},
	{
		text = "Show specialization name",
		func = function() DualSpeccer.db.profile.showSpecName = not DualSpeccer.db.profile.showSpecName; DualSpeccer:Update() end,
		checked = function() return DualSpeccer.db.profile.showSpecName end,
		tooltipTitle = "Show specialization name",
		tooltipText = "Displays the name of the specialization instead of primary/secondary."
	}
}

-- Match patterns we use. Replace any "." with "%." and any "%s" with "(.*)".
local spamFilterMatch1 = string.gsub(ERR_LEARN_ABILITY_S:gsub('%.', '%.'), '%%s', '(.*)')
local spamFilterMatch2 = string.gsub(ERR_LEARN_SPELL_S:gsub('%.', '%.'), '%%s', '(.*)')
local spamFilterMatch3 = string.gsub(ERR_SPELL_UNLEARNED_S:gsub('%.', '%.'), '%%s', '(.*)')

-- Make some commonly used functions local.
local GetNumTalentGroups, GetActiveTalentGroup = GetNumTalentGroups, GetActiveTalentGroup

DualSpeccerLDB = LibStub("LibDataBroker-1.1"):NewDataObject("DualSpeccer", {
	type = "data source",
	text = "DualSpeccer",
	icon = "Interface\\Icons\\Ability_Marksmanship",
	tocname = "DualSpeccer"
})

function DualSpeccer:OnInitialize()
	local dbDefaults = {
		profile = {
			filterSpam = true,
			showSpecName = false
		}
	}
	self.db = LibStub("AceDB-3.0"):New("DualSpeccerDB", dbDefaults, "Default")
end

function DualSpeccer:OnEnable()
	self:RegisterEvent("UNIT_SPELLCAST_START")
	self:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED")
	self:RegisterEvent("PLAYER_LEAVING_WORLD")
	
	-- Chances are good that the P_E_W will fire after we're done, registering PLAYER_TALENT_UPDATE twice. This
	-- is more of an AceAddon implementation thing and CallbackHandler overwrites it anyway.
	self:RegisterEvent("PLAYER_ENTERING_WORLD")
	self:RegisterEvent("PLAYER_TALENT_UPDATE")
	
	-- The important part here is that it happens after the first OnUpdate which is seemingly the time when correct
	-- information becomes available.
	self:ScheduleTimer("Update", 0.01)
end

function DualSpeccer:OnDisable()
	
end

-- A "corrupted" talent update fires between LEAVING and ENTERING when zoning. We don't want this one.
function DualSpeccer:PLAYER_LEAVING_WORLD(event)
	self:UnregisterEvent("PLAYER_TALENT_UPDATE")
end

function DualSpeccer:PLAYER_ENTERING_WORLD(event)
	self:RegisterEvent("PLAYER_TALENT_UPDATE")
end

function DualSpeccer:ACTIVE_TALENT_GROUP_CHANGED(event, ...)
	local newGroup = ...
	self:Print(("Switched to %s talent spec."):format(groupNames[newGroup]))
end

function DualSpeccer:UNIT_SPELLCAST_START(event, ...)
	local unit, spellName = ...
	if unit == "player" and (spellName == primarySpecSpellName or spellName == secondarySpecSpellName) then
		lastSwitchStartTime = GetTime()
	end
end

function DualSpeccer:PLAYER_TALENT_UPDATE(event)
	self:Update()
end

-- Our chat filter function.
local function filter(self, event, msg)
	-- You have unlearned Blahblah.
	-- You have learned a new spell: Blahblah.
	-- You have learned a new ability: Blahblah.
	if (lastSwitchStartTime > GetTime() - 8) and (msg:find(spamFilterMatch1) ~= nil or msg:find(spamFilterMatch2) ~= nil or msg:find(spamFilterMatch3) ~= nil) then
		return true
	end
end

-- Updates our talent information.
function DualSpeccer:Update()
	for i = 1, GetNumTalentGroups() do
		specCache[i] = specCache[i] or {}
		local thisCache = specCache[i]
		-- Let the Blizzard code do all of the dirty work. This also ensures that it's consistent with whatever Blizzard says.
		TalentFrame_UpdateSpecInfoCache(thisCache, false, false, i)
		if thisCache.primaryTabIndex and thisCache.primaryTabIndex ~= 0 then
			thisCache.specName = thisCache[thisCache.primaryTabIndex].name
			thisCache.mainTabIcon = thisCache[thisCache.primaryTabIndex].icon
		else
			if thisCache.totalPointsSpent == 0 then
				thisCache.specName = "None"
			else
				thisCache.specName = "Hybrid"
			end
			thisCache.mainTabIcon = "Interface\\Icons\\Ability_Marksmanship"
		end
		thisCache.specGroupName = groupNamesCaps[i]
	end
	
	local activeGroupNum = GetActiveTalentGroup()
	local curCache = specCache[activeGroupNum]
	
	if self.db.profile.showSpecName then
		DualSpeccerLDB.text = curCache.specName
	else
		DualSpeccerLDB.text = curCache.specGroupName
	end
	
	DualSpeccerLDB.icon = curCache.mainTabIcon
	
	if not filtering and self.db.profile.filterSpam then
		ChatFrame_AddMessageEventFilter("CHAT_MSG_SYSTEM", filter)
	elseif filtering and not self.db.profile.filterSpam then
		ChatFrame_RemoveMessageEventFilter("CHAT_MSG_SYSTEM", filter)
	end
end

-- Menu anchoring stuff stolen from ClosetGnome.
local menuPopupFrame = CreateFrame("Frame", "DualSpeccerMenu", UIParent, "UIDropDownMenuTemplate")
function DualSpeccerLDB.OnClick(displayFrame, button)
	button = button or "LeftButton"
	
	if button == "LeftButton" then
		if GetNumTalentGroups() < 2 then
			DualSpeccer:Print("This character does not have a second specialization.")
			return
		end
		
		local curSpec = GetActiveTalentGroup()
		-- -1 + 3 = 2 and -2 + 3 = 1.
		local newSpec = -curSpec + 3
		SetActiveTalentGroup(newSpec)
	elseif button == "RightButton" then
		EasyMenu(rightClickMenu, menuPopupFrame, displayFrame, 20, 4, "MENU")
	end
end

function DualSpeccerLDB.OnTooltipShow(tooltip)
	tooltip:AddLine("DualSpeccer")
	local curGroup = GetActiveTalentGroup()
	local curCache = specCache[curGroup]
	
	if not curCache then
		DualSpeccer:Update()
		curCache = specCache[curGroup]
		if not curCache then
			return
		end
	end
	
	tooltip:AddDoubleLine("Active spec", ("%s (%s)"):format(curCache.specGroupName, curCache.specName))
	
	-- We're showing the tooltip while our data is bad. Just don't show the spec info.
	if not specCache[1][1] or specCache[1][1].pointsSpent == nil then
		return
	end
	
	-- Greenish if selected, yellowish white if not selected.
	local primSpecColor = "ffff99"
	if curGroup == 1 then primSpecColor = "6adb54" end
	
	tooltip:AddDoubleLine("Primary spec", ("|cff%s%s|r (%d/%d/%d)"):format(primSpecColor, specCache[1].specName, specCache[1][1].pointsSpent, specCache[1][2].pointsSpent, specCache[1][3].pointsSpent))
	if GetNumTalentGroups() >= 2 then
		local secSpecColor = "ffff99"
		if curGroup == 2 then secSpecColor = "6adb54" end
		tooltip:AddDoubleLine("Secondary spec", ("|cff%s%s|r (%d/%d/%d)"):format(secSpecColor, specCache[2].specName, specCache[2][1].pointsSpent, specCache[2][2].pointsSpent, specCache[2][3].pointsSpent))
		tooltip:AddLine(" ")
		tooltip:AddLine("|cffeda55fClick|r to switch to your other specialization.")
		tooltip:AddLine("|cffeda55fRight-Click|r to open the menu.")
	end
end

